package de.meinefirma.meinprojekt;

import java.util.stream.Stream;
import org.junit.jupiter.api.*;
import org.junit.platform.runner.JUnitPlatform;
import org.junit.runner.RunWith;
import static org.junit.jupiter.api.Assertions.*;

@RunWith( JUnitPlatform.class )
public class AppTest
{
   @BeforeAll
   static void beforeAll() {
      System.out.println( "Vor allen Tests ..." );
   }

   @AfterAll
   static void afterAll() {
      System.out.println( "... fertig" );
   }

   @BeforeEach
   void beforeEach() {
      System.out.println( "Vor jedem einzelnen Test ..." );
   }

   @Test @DisplayName( "Teste App.main()" )
   void testeAppMain() {
      App.main( new String[] { "7", "3" } );
   }

   @Test @DisplayName( "Teste 3 / 2" )
   void testeCalc() {
      System.out.println( "3 / 2 ..." );
      assertEquals( 1, App.calc( 3, 2 ), "Ergebnis muss 1 sein" );
   }

   @Test @DisplayName( "Teste Exception nach Division by zero" )
   void testeExeption() {
      System.out.println( "1 / 0 ..." );
      App app = new App();
      ArithmeticException ex = assertThrows( ArithmeticException.class, () -> App.calc(1,0) );
      assertEquals( "/ by zero", ex.getMessage(), "Exception-Message" );
   }

   @Test @DisplayName( "Mehrere Tests zusammengefasst" )
   void testeAssertAll() {
      System.out.println( "20 / ?? ..." );
      assertAll( "Mehrere Tests",
            () -> assertEquals( 6, App.calc(20, 3 ) ),
            () -> assertEquals( 5, App.calc(20, 4 ) ),
            () -> assertEquals( 4, App.calc(20, 5 ) ) );
   }

   @TestFactory @DisplayName( "DynamicTest mit TestFactory" )
   Stream<DynamicTest> testeDynamicTest() {
      System.out.println( "1, 2, 3 / 2 ..." );
      return Stream.of( 1, 2, 3 ).map( i -> DynamicTest.dynamicTest( "i=" + i,
            () -> assertTrue( i > App.calc( i, 2 ) ) ) );
   }
}
